@@ -1,4 +1,5 @@
 import torch
+from plum import Union
 
 from . import B, dispatch
 from ..types import TorchNumeric, TorchDType, Int, TorchRandomState
@@ -70,19 +71,31 @@ def randn(dtype: TorchDType, *shape: Int):
 
 
 @dispatch
-def choice(state: TorchRandomState, a: TorchNumeric, n: Int):
-    choices = a[torch.randint(a.shape[0], (n,), generator=state)]
+def choice(
+    state: TorchRandomState,
+    a: TorchNumeric,
+    n: Int,
+    *,
+    p: Union[TorchNumeric, None] = None,
+):
+    if p is None:
+        p = torch.ones(a.shape[0], dtype=B.dtype_float(a), device=a.device)
+    choices = a[torch.multinomial(p, n, replacement=True, generator=state)]
     return state, choices[0] if n == 1 else choices
 
 
 @dispatch
-def choice(a: TorchNumeric, n: Int):
-    return choice(global_random_state(a), a, n)[1]
+def choice(a: TorchNumeric, n: Int, *, p: Union[TorchNumeric, None] = None):
+    return choice(global_random_state(a), a, n, p=p)[1]
 
 
 @dispatch
 def randint(
-    state: TorchRandomState, dtype: TorchDType, *shape: Int, lower: Int = 0, upper: Int
+    state: TorchRandomState,
+    dtype: TorchDType,
+    *shape: Int,
+    lower: Int = 0,
+    upper: Int,
 ):
     dtype = B.dtype_int(dtype)
     return state, torch.randint(
